Loading...
include/mpool.h Libc-391.4.2 Libc-262
--- Libc/Libc-391.4.2/include/mpool.h
+++ Libc/Libc-262/include/mpool.h
@@ -1,27 +1,26 @@
 /*
- * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 /*-
- * Copyright (c) 1991, 1993, 1994
+ * Copyright (c) 1991, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -52,51 +51,56 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)mpool.h	8.2 (Berkeley) 7/14/94
- * $FreeBSD: src/include/mpool.h,v 1.9 2002/03/23 17:24:53 imp Exp $
+ *	@(#)mpool.h	8.1 (Berkeley) 6/2/93
  */
 
-#ifndef _MPOOL_H_
-#define _MPOOL_H_
-
-#include <sys/queue.h>
-
 /*
- * The memory pool scheme is a simple one.  Each in-memory page is referenced
- * by a bucket which is threaded in up to two of three ways.  All active pages
- * are threaded on a hash chain (hashed by page number) and an lru chain.
- * Inactive pages are threaded on a free chain.  Each reference to a memory
- * pool is handed an opaque MPOOL cookie which stores all of this information.
+ * The memory pool scheme is a simple one.  Each in memory page is referenced
+ * by a bucket which is threaded in three ways.  All active pages are threaded
+ * on a hash chain (hashed by the page number) and an lru chain.  Inactive
+ * pages are threaded on a free chain.  Each reference to a memory pool is
+ * handed an MPOOL which is the opaque cookie passed to all of the memory
+ * routines.
  */
 #define	HASHSIZE	128
 #define	HASHKEY(pgno)	((pgno - 1) % HASHSIZE)
 
-/* The BKT structures are the elements of the queues. */
-typedef struct _bkt {
-	TAILQ_ENTRY(_bkt) hq;		/* hash queue */
-	TAILQ_ENTRY(_bkt) q;		/* lru queue */
-	void    *page;			/* page */
-	pgno_t   pgno;			/* page number */
+/* The BKT structures are the elements of the lists. */
+typedef struct BKT {
+	struct BKT	*hnext;		/* next hash bucket */
+	struct BKT	*hprev;		/* previous hash bucket */
+	struct BKT	*cnext;		/* next free/lru bucket */
+	struct BKT	*cprev;		/* previous free/lru bucket */
+	void		*page;		/* page */
+	pgno_t		pgno;		/* page number */
 
 #define	MPOOL_DIRTY	0x01		/* page needs to be written */
 #define	MPOOL_PINNED	0x02		/* page is pinned into memory */
-	u_int8_t flags;			/* flags */
+	unsigned long	flags;		/* flags */
 } BKT;
 
+/* The BKTHDR structures are the heads of the lists. */
+typedef struct BKTHDR {
+	struct BKT	*hnext;		/* next hash bucket */
+	struct BKT	*hprev;		/* previous hash bucket */
+	struct BKT	*cnext;		/* next free/lru bucket */
+	struct BKT	*cprev;		/* previous free/lru bucket */
+} BKTHDR;
+
 typedef struct MPOOL {
-	TAILQ_HEAD(_lqh, _bkt) lqh;	/* lru queue head */
-					/* hash queue array */
-	TAILQ_HEAD(_hqh, _bkt) hqh[HASHSIZE];
-	pgno_t	curcache;		/* current number of cached pages */
-	pgno_t	maxcache;		/* max number of cached pages */
-	pgno_t	npages;			/* number of pages in the file */
-	unsigned long	pagesize;	/* file page size */
-	int	fd;			/* file descriptor */
-					/* page in conversion routine */
-	void    (*pgin)(void *, pgno_t, void *);
-					/* page out conversion routine */
-	void    (*pgout)(void *, pgno_t, void *);
-	void	*pgcookie;		/* cookie for page in/out routines */
+	BKTHDR	free;			/* The free list. */
+	BKTHDR	lru;			/* The LRU list. */
+	BKTHDR	hashtable[HASHSIZE];	/* Hashed list by page number. */
+	pgno_t	curcache;		/* Current number of cached pages. */
+	pgno_t	maxcache;		/* Max number of cached pages. */
+	pgno_t	npages;			/* Number of pages in the file. */
+	u_long	pagesize;		/* File page size. */
+	int	fd;			/* File descriptor. */
+					/* Page in conversion routine. */
+	void    (*pgin) __P((void *, pgno_t, void *));
+					/* Page out conversion routine. */
+	void    (*pgout) __P((void *, pgno_t, void *));
+	void	*pgcookie;		/* Cookie for page in/out routines. */
 #ifdef STATISTICS
 	unsigned long	cachehit;
 	unsigned long	cachemiss;
@@ -110,18 +114,43 @@
 #endif
 } MPOOL;
 
+#ifdef __MPOOLINTERFACE_PRIVATE
+/* Macros to insert/delete into/from hash chain. */
+#define rmhash(bp) { \
+        (bp)->hprev->hnext = (bp)->hnext; \
+        (bp)->hnext->hprev = (bp)->hprev; \
+}
+#define inshash(bp, pg) { \
+	hp = &mp->hashtable[HASHKEY(pg)]; \
+        (bp)->hnext = hp->hnext; \
+        (bp)->hprev = (struct BKT *)hp; \
+        hp->hnext->hprev = (bp); \
+        hp->hnext = (bp); \
+}
+
+/* Macros to insert/delete into/from lru and free chains. */
+#define	rmchain(bp) { \
+        (bp)->cprev->cnext = (bp)->cnext; \
+        (bp)->cnext->cprev = (bp)->cprev; \
+}
+#define inschain(bp, dp) { \
+        (bp)->cnext = (dp)->cnext; \
+        (bp)->cprev = (struct BKT *)(dp); \
+        (dp)->cnext->cprev = (bp); \
+        (dp)->cnext = (bp); \
+}
+#endif
+
 __BEGIN_DECLS
-MPOOL	*mpool_open(void *, int, pgno_t, pgno_t);
-void	 mpool_filter(MPOOL *, void (*)(void *, pgno_t, void *),
-	    void (*)(void *, pgno_t, void *), void *);
-void	*mpool_new(MPOOL *, pgno_t *);
-void	*mpool_get(MPOOL *, pgno_t, unsigned int);
-int	 mpool_put(MPOOL *, void *, unsigned int);
-int	 mpool_sync(MPOOL *);
-int	 mpool_close(MPOOL *);
+MPOOL	*mpool_open __P((DBT *, int, pgno_t, pgno_t));
+void	 mpool_filter __P((MPOOL *, void (*)(void *, pgno_t, void *),
+	    void (*)(void *, pgno_t, void *), void *));
+void	*mpool_new __P((MPOOL *, pgno_t *));
+void	*mpool_get __P((MPOOL *, pgno_t, u_int));
+int	 mpool_put __P((MPOOL *, void *, u_int));
+int	 mpool_sync __P((MPOOL *));
+int	 mpool_close __P((MPOOL *));
 #ifdef STATISTICS
-void	 mpool_stat(MPOOL *);
+void	 mpool_stat __P((MPOOL *));
 #endif
 __END_DECLS
-
-#endif